/**************** (c) 2002  STMicroelectronics **********************
     
PROJECT  :
COMPILER : 

MODULE  :  blink.c
VERSION :  

CREATION DATE : 01.2001 

AUTHOR : Pavel Koutensky 

-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-

DESCRIPTION :   Blinking feature
              
-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-

MODIFICATIONS :

 ******************************************************************************
 THE SOFTWARE INCLUDED IN THIS FILE IS FOR GUIDANCE ONLY. ST MICROELECTRONICS
 SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES
 WITH RESPECT TO ANY CLAIMS ARISING FROM USE OF THIS SOFTWARE.
 ******************************************************************************

******************************************************************************/

/* Include files */

#include "lib.h"        /* Predifined libraries working at bit level */
#include "debug.h"      /* Debugging options */
#include "spi_hr.h" 
#include "ports_hr.h" 
#include "adc_8bit.h"
//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-
//----------------------------variables-----------------------
u8 static mode = 5;
u8 static blink_data1 = 0, blink_data2 = 0; 
u8 static counter_SW = 0;	
u8 static DATA_RED = 1;    // value between 0 to 31 - after reset start with minimum bright in order to have right start up (input voltage ripple)
u8 static DATA_GREEN = 1;  // value between 0 to 31
u8 static DATA_BLUE = 1;    // value between 0 to 31
u8 static first_time = ON;
u8 static end_of_write=OFF;
u8 static time_to_write=ON;
//-------------------------Seting blinking modes - ei1 interrupt in ports.c------------
u8 Read_mode(void)
{
return mode;
}
//----------------------------------------------------------------------------
void Set_mode(u8 m)
{
mode = m;
}
//--------------------------nastaveni rychlosti blikani ------
void Time_to_write(void)  // SW PWM generation - time base 
{
if (end_of_write)     // all data must be sent it is only protection
   {                   
   end_of_write =OFF;
   time_to_write= ON;
   }
}
//---------------------prenos dat po SPI-----------------------------------*/
void Send_DATA(void)
{
u8 static counter_SPI = 0;

if (first_time)
{
counter_SPI++;
switch (counter_SPI) {
   case 1 : {
             SPISR;
             SPIDR=blink_data2;
             }break;
   default : {                                //konec vysilani dat po SPI
   						SPISR;
   						SPIDR;
              SetBit(PADR,0);  // latching data in STP04CM596  
              SetBit(PADR,1);  // latching data in STP08CL596
              ClrBit(PADR,0);
							ClrBit(PADR,1);
   						end_of_write = ON;
   						counter_SPI = 0;
   						first_time = OFF;
							}
   						break;
   }
}
else
{
SPISR;
SPIDR;
SetBit(PADR,0);  // latching data in STP04CM596  
ClrBit(PADR,0);
end_of_write = ON;
}
}
//------------------ SW modulation -------------------------------------
void SW_modulation(void)
{
if (DATA_RED == counter_SW) {ClrBit(blink_data2,4); ClrBit(blink_data2,0);}
if (DATA_GREEN == counter_SW) 
  {
	ClrBit(blink_data2,6); ClrBit(blink_data2,5);
	ClrBit(blink_data2,2); ClrBit(blink_data2,1);
	}
if (DATA_BLUE == counter_SW) {ClrBit(blink_data2,7); ClrBit(blink_data2,3);}
}

//------------------blikaci mody----------------------------------------
void blink(void)
{
u8 static old_mode = 0;	
u8 static timing = 0;  // how fast automatic color is changed
u8 static manual_color = 1;  // different color can be set through this value
u8 static bright = 63;
u8 static counter_time = 0;  //for temperature delay (the temperature is measured after each 1.6s and corrected)
u8 static refresh_signal = ON;      // for turning ON and OFF over temperature LED 
u8 static D6;
u8 static start_up = ON;              // slow start up
u8 static PWM_bright = 63;
u8 static set_bright_pot = ON;   // the brightness is decreased if overtemperature occured

//----------------- Temperature limits for the STLM20 temperature sensor ------------
u8 const MAX_TEMPERATURE = 65;   // 50C => Vout = 1.263V on ADC => 65  see datasheet
//u8 const MAX_TEMPERATURE = 59; // 60C => Vout = 1.145V on ADC => 59  ADC=256*Vout/Vc = 256*Vout/5
//u8 const MAX_TEMPERATURE = 53; // 70C => Vout = 1.027V on ADC => 53
//u8 const MAX_TEMPERATURE = 47; // 80C => Vout = 0.909V on ADC => 47

//---------- Temperature limits for the OSTAR module NTC temperature sensor ------------
//u8 const MAX_TEMPERATURE = 109; // 50C => Rntc = 3k5 => Vout = 2.13V on ADC => 109  see datasheet
//u8 const MAX_TEMPERATURE = 89;  // 60C => Rntc = 5k5 => Vout = 1.73V on ADC => 89  ADC=256*Vout/Vc = 256*Vout/5
//u8 const MAX_TEMPERATURE = 68;  // 70C => Rntc = 1k7 => Vout = 1.32V on ADC => 68
//u8 const MAX_TEMPERATURE = 55;  // 80C => Rntc = 1k3 => Vout = 1.08V on ADC => 55
//NOTE: if MAX_TEMPERATURE = 65 the maximum temperature on OSTAR is approx. 72C

// ---------------------blink function starts here ----------------------
  while (!time_to_write);   // time base generated from PWM AR TIMER
	time_to_write = OFF;                             // all diodes are latched on
//--------------------Brightness PWM regulation ---------------------
if (counter_SW == 0) {PWM_bright = (Read_PWM_brightness() >> 2);} // set color after each 38,4ms (4x6.4ms)
if (set_bright_pot) {bright = PWM_bright;}

//-------------------- temperature regulation -------------------------
	if (counter_time == 255)           // counter time is incremented after one period of SW PWM signal
    { 	
		if (MAX_TEMPERATURE > Read_temperature()) // negative voltage coeficient see STLM20 datasheet
			{
		  if (bright > 0) {bright--;}
			if (bright > PWM_bright)                    // if the brightness from pot. is lower than automatic bright decreasing it is used
			   {bright = PWM_bright; set_bright_pot = ON;}
		     else
			   {set_bright_pot = OFF;}  				 
			if (refresh_signal == OFF) 
			    {
			    refresh_signal = ON; 
					first_time = ON; 
					D6 = 0x20;
					}             // turn ON over temperature (light)
			}
			else
			{
			bright++;
			//if (bright == 64) {bright = 63;}
		  if (bright > PWM_bright)                     // increase brightness to level sets by potentiometer  
			   {bright = PWM_bright; set_bright_pot = ON;}  
			   else
			   {set_bright_pot = OFF;}
			if (refresh_signal) 
			   {
				 refresh_signal = OFF; 
				 first_time = ON; 
				 D6 = 0x00;
				 }               // turn OFF over temperature (light)
			}
	  counter_time = 0;
		}
  if (counter_time == 20)  {start_up = OFF;}         // LEDs will bright after 130ms}
	if (counter_SW == 62) {counter_time++;}
	
  //--------------PWM brightness regulation OE -pin LED ------------------------- 	
	   if ((counter_SW == 0)&&(bright > 0)) {ClrBit(PADR,2);}           // setting PWM modulation on pin OE for temperature regulation  
	   if ((counter_SW == bright)||(bright == 0)) {SetBit(PADR,2);}     // for no brightness set PADR2 always
	
//------------------------------------------------------------------------		
	if (old_mode != mode) 
		{
		old_mode = mode;
	  if (old_mode == 1) {DATA_BLUE = 0; DATA_RED = 0; DATA_GREEN = 0;}
		if (old_mode == 5) {DATA_BLUE = 63; DATA_RED = 0; DATA_GREEN = 0;}
		if (old_mode == 6) {DATA_BLUE = 63; DATA_RED = 63; DATA_GREEN = 63;}
		first_time = ON;
		}
											 
	switch (old_mode) {
		case 1 : {
					   blink_data1 = 0x01;
						 if (counter_SW == 0) {
						                      // if (start_up) 
																	//     {
																	//		 while (MAX_TEMPERATURE > Read_temperature());  //wait, because the LEDs are too hot
																	//		 DATA_RED = 1; 
																	//	   }
																	// else
                                  //     {
																			 DATA_RED = (Read_color_brightness() >> 2); 
																	//	   } 																			 
																	 blink_data2 = 0xFF;
																	}                                     // valid value are from 0 to 31 for DATA_RED
						 SW_modulation();
						 }break;
		case 2 : {
		         blink_data1 = 0x02;
						 if (counter_SW == 0) {DATA_GREEN = (Read_color_brightness() >> 2); blink_data2 = 0xFF;} // valid value are from 0 to 31 for DATA_GREEN
						 SW_modulation();
		         }break;
    case 3 : {
		         blink_data1 = 0x04;
						 if (counter_SW == 0) {DATA_BLUE = (Read_color_brightness() >> 2); blink_data2 = 0xFF;} // valid value are from 0 to 31 for DATA_BLUE
						 SW_modulation();
		         }break;
    case 4 : {blink_data1 = 0x08;	
							if (counter_SW == 0)
								{
								blink_data2 = 0xFF;	
								manual_color = Read_color_brightness();        // manual color is between 1 to 254
							  if (manual_color > 252) {manual_color = 252;}  // max brightness is 42 in this case
							                                                 
								if ((manual_color >= 1)&&(manual_color <= 42)) {DATA_BLUE = 42; DATA_RED = 0; DATA_GREEN = manual_color;}
								if ((manual_color >= 43)&&(manual_color <= 84)) {DATA_GREEN = 42; DATA_RED = 0; DATA_BLUE = (85 - manual_color);}
								if ((manual_color >= 85)&&(manual_color <= 126)) {DATA_GREEN = 42; DATA_BLUE = 0; DATA_RED = (manual_color - 84);}
								if ((manual_color >= 127)&&(manual_color <= 168)) {DATA_BLUE = 0; DATA_RED = 42; DATA_GREEN = (169 - manual_color);}
								if ((manual_color >= 169)&&(manual_color <= 210)) {DATA_GREEN = 0; DATA_RED = 42; DATA_BLUE = (manual_color - 168);}
								if ((manual_color >= 211)&&(manual_color <= 252)) {DATA_GREEN = 0; DATA_BLUE = 42; DATA_RED = (253 - manual_color);}
							  }
							SW_modulation();
             }break;
    case 5 : {
		          blink_data1 = 0x10;	
		          if (counter_SW == 0)
								{
								if (start_up) 
									{
									while (MAX_TEMPERATURE > Read_temperature());  //wait, because the LEDs are too hot
									}
                  else
                  {									
									blink_data2 = 0xFF;
									timing++;
									if (timing == 2)
										{							
										if ((DATA_BLUE == 63)&&(DATA_RED == 0)) {DATA_GREEN++;}
										if ((DATA_BLUE == 0)&&(DATA_GREEN == 63)) {DATA_RED++;}
										if ((DATA_GREEN == 63)&&(DATA_RED == 0)) {DATA_BLUE--;}
										if ((DATA_GREEN == 0)&&(DATA_RED == 63)) {DATA_BLUE++;}
										if ((DATA_RED == 63)&&(DATA_BLUE == 0)) {DATA_GREEN--;}
										if ((DATA_BLUE == 63)&&(DATA_GREEN == 0)) {DATA_RED--;}
										timing = 0;
										}
								  }							
								}
             SW_modulation();
						 }break;
    case 6 : {blink_data1 = 0x40;	
		         if (counter_SW == 0) {blink_data2 = 0xFF;} // start point for SW PWM modulation
						 SW_modulation();
             }break;						 
	default : {}break;      
	}
	
	counter_SW++;
  if (counter_SW == 64) {counter_SW = 0;} 
if (first_time) {
                 blink_data1 = (blink_data1 | D6); 
								 SPIDR= blink_data1;
								}
  else {SPIDR = blink_data2;}
}  

/*** (c) 2002  STMcroelectronics ****************** END OF FILE ***/
